import pytest
from pygeppetto.constants import UserPrivileges
from pygeppetto.local_data_model import LocalGeppettoProject, LocalUser, \
    LocalUserGroup, \
    LocalExperiment
from pygeppetto.managers import GeppettoManager
from pygeppetto.managers.geppetto_manager import GeppettoExecutionException, \
    GeppettoAccessException, Scope
from pygeppetto.model import GeppettoModel


@pytest.fixture
def manager():
    GeppettoManager._instances = {}
    return GeppettoManager()


@pytest.fixture
def mock__model():
    return GeppettoModel(name='fake_model')


def test__geppettomanager_user_reaffectation(manager, mock__model):
    user1 = LocalUser(id=1, name='user1', group=None)
    user2 = LocalUser(id=2, name='user2', group=None)
    manager.user = user1

    with pytest.raises(GeppettoExecutionException):
        manager.user = user2


def test__geppettomanager_load_project(manager, mock__model):
    project = LocalGeppettoProject(name='TestProject', id=1, geppetto_model=mock__model, volatile=True)
    project.volatile = True

    assert manager.is_project_open(project) is False
    manager.load_project(project)
    assert manager.is_project_open(project) is True

    # with pytest.raises(GeppettoExecutionException):
    #     manager.load_project(project)

    project = LocalGeppettoProject(name='TestProject2', id=2, geppetto_model=mock__model, volatile=True)
    manager.scope = Scope.RUN
    manager.load_project(project)
    assert manager.is_project_open(project) is True


def test__geppettomanager_load_project_access_rights(manager, mock__model):
    project = LocalGeppettoProject(name='TestProject', id=1, geppetto_model=mock__model, volatile=False)

    with pytest.raises(GeppettoAccessException):
        manager.load_project(project)

    group = LocalUserGroup(name='TestGroup',
                           privileges=(UserPrivileges.READ_PROJECT,))
    manager.user = LocalUser(id=1, name='TestUser', projects=(project,),
                             group=group)
    manager.load_project(project)
    assert manager.is_project_open(project)


def test__geppettomanager_load_access_no_rights(manager, mock__model):
    project = LocalGeppettoProject(name='TestProject', id=1, geppetto_model=mock__model, volatile=False)

    group = LocalUserGroup(name='TestGroup')
    manager.user = LocalUser(id=1, name='TestUser', projects=(project,),
                             group=group)
    with pytest.raises(GeppettoAccessException):
        manager.load_project(project)

    group.privileges.append(UserPrivileges.READ_PROJECT)
    del manager.user.projects[0]
    with pytest.raises(GeppettoAccessException):
        manager.load_project(project)


def test__geppettomanager_get_runtime_project(manager, mock__model):
    manager.scope = Scope.RUN

    project = LocalGeppettoProject(name='TestProject', id=1, geppetto_model=mock__model, volatile=True)
    manager.load_project(project)
    runtime = manager.get_runtime_project(project)
    assert runtime.project is project
    assert project in manager.opened_projects

    project = LocalGeppettoProject(name='TestProject2', id=2, geppetto_model=mock__model, volatile=True)
    runtime = manager.get_runtime_project(project)
    assert runtime.project is project
    assert project in manager.opened_projects

    with pytest.raises(GeppettoExecutionException):
        project = LocalGeppettoProject(name='TestProject3', id=3, geppetto_model=mock__model, volatile=False)
        manager.scope = Scope.CONNECTION
        manager.get_runtime_project(project)


def test__geppettomanager_is_user_project(manager, mock__model):
    project = LocalGeppettoProject(name='TestProject', id=1, geppetto_model=mock__model, volatile=True)
    project.id = 123456789
    manager.user = LocalUser(id=1, name='TestUser', projects=(project,),
                             group=LocalUserGroup('TestGroup'))

    assert manager.is_user_project(project) is True
    assert manager.is_user_project(project.id) is True

    project = LocalGeppettoProject(name='TestProject2', id=2, geppetto_model=mock__model, volatile=True)
    project.id = 456789123
    assert manager.is_user_project(project) is False
    assert manager.is_user_project(project.id) is False


def test__geppettomanager_close_project(manager, mock__model):
    project = LocalGeppettoProject(name='TestProject', id=1, geppetto_model=mock__model, volatile=True)
    project.volatile = True

    manager.load_project(project)
    assert manager.is_project_open(project) is True

    manager.close_project(project)
    assert manager.is_project_open(project) is False

    project = LocalGeppettoProject(name='TestProject2', id=2, geppetto_model=mock__model, volatile=True)
    with pytest.raises(GeppettoExecutionException):
        manager.close_project(project)


def test__geppettomanager_load_experiment(manager, mock__model):
    project = LocalGeppettoProject(name='TestProject', id=1, geppetto_model=mock__model, volatile=True)
    experiment = LocalExperiment(name='TestExperiment', project=project)

    group = LocalUserGroup(name='TestGroup')
    manager.user = LocalUser(id=1, name='TestUser', projects=(project,),
                             group=group)

    with pytest.raises(GeppettoAccessException):
        manager.load_experiment(experiment)

    group.privileges.append(UserPrivileges.READ_PROJECT)

    with pytest.raises(GeppettoExecutionException):
        manager.load_experiment(experiment)

    manager.load_project(project)
    state = manager.load_experiment(experiment)

    assert state is not None



import json
import pytest

from pygeppetto.model import GeppettoLibrary
from pygeppetto.model.model_access import GeppettoModelAccess
from pygeppetto.model.datasources.datasources import DataSource, QueryResults, QueryResult
from pygeppetto.model.datasources import BooleanOperator
from pygeppetto.services.data_source_service import DataSourceService
from pygeppetto.services.model_interpreter import add_model_interpreter
from pygeppetto.services.data_source.neo4j import Neo4jDataSourceService
from .mocks import MockModelInterpreter, neo4j_response as mock_neo4j_response

def create_result(id): 
    return (id, f"Result number {id}")

def create_query_result(id):
    return QueryResult(values=create_result(id))
    
def model_access():
    model_interpreter = MockModelInterpreter()
    model_library = GeppettoLibrary(name='mocklibrary', id='mocklibrary')
    geppetto_model = model_interpreter.create_model(library=model_library)
    add_model_interpreter(model_library.id, model_interpreter)
    return GeppettoModelAccess(geppetto_model)


@pytest.fixture
def data_source_service():
    return DataSourceService(configuration=DataSource(),
                             model_access=model_access())


def test_datasource_service_get_results(data_source_service):
    header = ["ID", "content"]

    results = {}
    assert len(data_source_service.get_results(results).results) == 0

    # OR [0, 1]  == [0, 1]
    query_results = [ create_query_result(i) for i in range(2) ]
    query_results = QueryResults(id="1", header=header, results=query_results)
    results[query_results] = BooleanOperator.OR

    assert len(data_source_service.get_results(results).results) == 2

    # OR [1, 2, 3] == [0, 1, 2, 3]
    query_results = [ create_query_result(i+1) for i in range(3) ]
    query_results = QueryResults(id="2", header=header, results=query_results)
    results[query_results] = BooleanOperator.OR

    assert len(data_source_service.get_results(results).results) == 4

    # AND [0, 1]  ==  [0, 1]
    query_results = [ create_query_result(i) for i in range(2) ]
    query_results = QueryResults(id="3", header=header, results=query_results)
    results[query_results] = BooleanOperator.AND

    assert len(data_source_service.get_results(results).results) == 2

    # NAND [0] == [1]
    query_results = [ create_query_result(0) ]
    query_results = QueryResults(id="4", header=header, results=query_results)
    results[query_results] = BooleanOperator.NAND

    assert len(data_source_service.get_results(results).results) == 1
    assert data_source_service.get_results(results).results[0].values[1] == "Result number 1"

def test_neo4j_datasource_service():
    neo4j_response = json.dumps(mock_neo4j_response())

    dss = Neo4jDataSourceService(configuration=DataSource(), model_access=model_access())
    
    query_results = dss.process_response([ neo4j_response ])
    
    assert query_results.header == ["ID", "n"]
    assert len(query_results.results) == 2
    assert query_results.results[0].values == ['0', '{"title": "The Matrix", "released": 1999}']


class GeppettoModelException(Exception): pass


class IllegalArgumentException(Exception): pass


class GeppettoExecutionException(Exception):
    pass


class GeppettoAccessException(Exception):
    pass


class GeppettoVisitingException(Exception): pass


class GeppettoInitializationException(Exception): pass


class ModelInterpreterException(Exception): pass

class GeppettoDataSourceException(Exception): pass


""" generated source for class URLReader """
SERVER_ROOT_TOKEN = "SERVER_ROOT"
from multimethod import multidispatch


# 
# 	 * @param urlString
# 	 * @param baseURL
# 	 * @return
#
@multidispatch
def getURL(urlString, baseURL):
    """ generated source for method getURL """
    url = urlString

    if baseURL != None:
        if not urlString.startsWith("http://") and not urlString.startsWith(
                "https://") and not urlString.startsWith(SERVER_ROOT_TOKEN):
            url = baseURL + urlString
    if urlString.startsWith("file:"):
        urlString.replace("file:", "")
    return getURL(url)


# 
# 	 * @param urlString
# 	 * @return
#
@getURL.register(object, str)
def getURL_0(urlString):
    """ generated source for method getURL_0 """
    url = None
    if urlString.startsWith("https://") or urlString.startsWith("http://"):
        url = urlString
    elif urlString.startsWith(SERVER_ROOT_TOKEN):
        url = urlString.replace(SERVER_ROOT_TOKEN, "")
    return url



import pytest
import pygeppetto.model.utils.pointer_utility as pointer_utility
from pygeppetto.constants import UserPrivileges
from pygeppetto.local_data_model import LocalGeppettoProject, LocalUser, LocalUserGroup
from pygeppetto.managers.geppetto_manager import GeppettoManager
from pygeppetto.model import GeppettoLibrary, CompositeType
from pygeppetto.model.types import ImportType
from pygeppetto.model.values import ImportValue
from pygeppetto.services.model_interpreter import add_model_interpreter

from .mocks import MockModelInterpreter
from pygeppetto.model.model_serializer import GeppettoModelSerializer


def create_geppetto_model():
    model_interpreter = MockModelInterpreter()
    model_library = GeppettoLibrary(name='mocklibrary', id='mocklibrary')
    geppetto_model = model_interpreter.create_model(library=model_library)
    add_model_interpreter(model_library.id, model_interpreter)
    return geppetto_model


@pytest.fixture
def geppetto_project():
    geppetto_model = create_geppetto_model()
    geppetto_project = LocalGeppettoProject(name='TestProject', experiments=None,
                                            geppetto_model=geppetto_model, id=None)
    geppetto_project.volatile = True
    return geppetto_project


geppetto_manager = None


def create_geppetto_manager():
    global geppetto_manager
    if geppetto_manager is None:
        geppetto_manager = GeppettoManager()

        group = LocalUserGroup('testgroup', privileges=[e for e in UserPrivileges])

        geppetto_manager.user = LocalUser(id=1, name='TestUser', projects=(geppetto_project,), group=group)

    return geppetto_manager


def test_import_value(geppetto_project):
    geppetto_model = geppetto_project.geppettoModel
    geppetto_manager = create_geppetto_manager()
    geppetto_manager.load_project(geppetto_project)
    assert ImportValue == type(pointer_utility.find_variable_from_path(geppetto_model, 'v3.v31').initialValues[0].value)

    GeppettoModelSerializer.serialize(geppetto_model, True)  # now the model is in sync
    model = geppetto_manager.resolve_import_value(path='v3.v31', geppetto_project=geppetto_project,
                                                  experiment=None)
    assert model is not None

    # assert not pointer_utility.find_variable_from_path(model=model, path='v3').eContainer().synched
    # print(GeppettoModelSerializer.serialize(model, True))
    assert not pointer_utility.find_variable_from_path(model=model, path='v3.v31').synched
    assert pointer_utility.find_variable_from_path(model=model, path='v1').synched
    assert pointer_utility.find_variable_from_path(model=model, path='v2').synched
    assert 4 == pointer_utility.find_variable_from_path(model=model, path='v3.v31').initialValues[0].value.value[0]

    # print(GeppettoModelSerializer.serialize(model, True))


def test_import_type(geppetto_project):
    geppetto_model = geppetto_project.geppettoModel

    geppetto_manager = create_geppetto_manager()
    geppetto_manager.load_project(geppetto_project)
    assert ImportType == type(
        pointer_utility.find_variable_from_path(geppetto_model, 'v4').types[0])  # autoresolve = False
    assert CompositeType == type(
        pointer_utility.find_variable_from_path(geppetto_model, 'v5').types[0])  # autoresolve = True

    GeppettoModelSerializer.serialize(geppetto_model, True)  # now the model is in sync
    model = geppetto_manager.resolve_import_type(type_paths=['mocklibrary.v4'], geppetto_project=geppetto_project)
    assert model is not None

    # assert not pointer_utility.find_variable_from_path(model=model, path='v3').eContainer().synched
    # print(GeppettoModelSerializer.serialize(model, True))
    assert pointer_utility.find_variable_from_path(model=model, path='v3.v31').synched
    assert pointer_utility.find_variable_from_path(model=model, path='v1').synched
    assert pointer_utility.find_variable_from_path(model=model, path='v2').synched
    assert pointer_utility.find_variable_from_path(model=model, path='v5').synched

    imported_type = pointer_utility.find_variable_from_path(model=model, path='v4').types[0]
    assert not imported_type.synched
    assert 'ct2' == imported_type.id
    assert 1 == len(imported_type.variables)
    assert imported_type.variables[0].initialValues[0].value.text == 'imported!!!'
    # assert GeppettoModelAccess().get_value('v4.vi') ==



from pygeppetto.model import GeppettoModel
from pygeppetto.model.exceptions import GeppettoModelException


class QueryNotFoundException(GeppettoModelException): pass


def get_query(query_path, model: GeppettoModel):
    data_source = None
    for token in query_path.split('.'):
        if data_source is None:
            try:
                return next(query for query in model.queries if query.id == token)
            except StopIteration:
                try:
                    data_source = next(ds for ds in model.dataSources if ds.id == token)
                except StopIteration:
                    raise QueryNotFoundException("Query `{}` not found in model.".format(query_path))
        else:
            try:
                return next(query for query in data_source.queries if query.id == token)
            except StopIteration:
                raise QueryNotFoundException("Query `{}` not found in model.".format(query_path))
    else:
        raise QueryNotFoundException("Query `{}` not found in model.".format(query_path))



from pygeppetto.model import Node, AQueryResult
from pygeppetto.model.datasources import Query


def query_check(query: Query, types=()) -> bool:
    """
    https://github.com/openworm/org.geppetto.core/blob/master/src/main/java/org/geppetto/core/datasources/QueryChecker.java)
    :return:
    """
    if len(query.matchingCriteria) == 0:
        return True

    all_types = set()
    all_types.update(types)

    for criterion in query.matchingCriteria:
        for type_to_match in criterion.type:
            if not type_to_match in all_types:
                for type_ in all_types:
                    if type_.extends_type(type_to_match):
                        return True
    return False


# Use to distinguish QueryResult by ID in a set
def modified_hash(self):
    return hash(self.values[self._id_pos])


def modified_eq(self, other):
    return self.values[self._id_pos] == other.values[self._id_pos]


# Restore default set identification
def regular_hash(self):
    return super(AQueryResult, self).__hash__()


def regular_eq(self, other):
    return super(AQueryResult, self).__eq__(other)


def set_custom_query_result_hash(id_index):
    AQueryResult._id_pos = id_index
    AQueryResult.__eq__ = modified_eq
    AQueryResult.__hash__ = modified_hash


def unset_custom_query_result_hash():
    AQueryResult._id_pos = None
    AQueryResult.__eq__ = regular_eq
    AQueryResult.__hash__ = regular_hash



import pytest

from pyecore.resources import ResourceSet, URI
from pygeppetto.model import GeppettoModel, model_utility

from .test_xmi import filepath

@pytest.fixture
def model() -> GeppettoModel:
    rset = ResourceSet()
    resource = rset.get_resource(URI(filepath('model_with_queries.xmi')))
    return resource.contents[0]


def test_get_query(model):
    query = model_utility.get_query('mockDataSource.mock_query', model)

    assert query.name == "A mock compound query"

    with pytest.raises(model_utility.QueryNotFoundException):
        model_utility.get_query('mockDataSource', model)



"""Definition of meta model 'datasources'."""
from functools import partial
import pyecore.ecore as Ecore
from pyecore.ecore import *
from ..model import Node, ISynchable


name = 'datasources'
nsURI = 'https://raw.githubusercontent.com/openworm/org.geppetto.model/development/src/main/resources/geppettoModel.ecore#//datasources'
nsPrefix = 'gep'

eClass = EPackage(name=name, nsURI=nsURI, nsPrefix=nsPrefix)

eClassifiers = {}
getEClassifier = partial(Ecore.getEClassifier, searchspace=eClassifiers)
BooleanOperator = EEnum('BooleanOperator', literals=['AND', 'NAND', 'OR'])


class DataSourceLibraryConfiguration(EObject, metaclass=MetaEClass):

    modelInterpreterId = EAttribute(eType=EString, derived=False, changeable=True)
    format = EAttribute(eType=EString, unique=True, derived=False, changeable=True)
    library = EReference(ordered=True, unique=True, containment=False)

    def __init__(self, library=None, modelInterpreterId=None, format=None, **kwargs):
        if kwargs:
            raise AttributeError('unexpected arguments: {}'.format(kwargs))

        super().__init__()

        if modelInterpreterId is not None:
            self.modelInterpreterId = modelInterpreterId

        if format is not None:
            self.format = format

        if library is not None:
            self.library = library


class QueryResults(EObject, metaclass=MetaEClass):

    id = EAttribute(eType=EString, unique=True, derived=False, changeable=True)
    header = EAttribute(eType=EString, unique=True, derived=False, changeable=True, upper=-1)
    results = EReference(ordered=True, unique=True, containment=True, upper=-1)

    def __init__(self, id=None, header=None, results=None, **kwargs):
        if kwargs:
            raise AttributeError('unexpected arguments: {}'.format(kwargs))

        super().__init__()

        if id is not None:
            self.id = id

        if header:
            self.header.extend(header)

        if results:
            self.results.extend(results)

    def getValue(self, field=None, row=None):

        raise NotImplementedError('operation getValue(...) not yet implemented')


class RunnableQuery(EObject, metaclass=MetaEClass):

    targetVariablePath = EAttribute(eType=EString, derived=False, changeable=True)
    queryPath = EAttribute(eType=EString, derived=False, changeable=True)
    booleanOperator = EAttribute(eType=BooleanOperator, derived=False,
                                 changeable=True, default_value=BooleanOperator.AND)

    def __init__(self, targetVariablePath=None, queryPath=None, booleanOperator=None, **kwargs):
        if kwargs:
            raise AttributeError('unexpected arguments: {}'.format(kwargs))

        super().__init__()

        if targetVariablePath is not None:
            self.targetVariablePath = targetVariablePath

        if queryPath is not None:
            self.queryPath = queryPath

        if booleanOperator is not None:
            self.booleanOperator = booleanOperator


@abstract
class AQueryResult(EObject, metaclass=MetaEClass):

    def __init__(self, **kwargs):
        if kwargs:
            raise AttributeError('unexpected arguments: {}'.format(kwargs))

        super().__init__()


class QueryMatchingCriteria(EObject, metaclass=MetaEClass):

    type = EReference(ordered=True, unique=True, containment=False, upper=-1)

    def __init__(self, type=None, **kwargs):
        if kwargs:
            raise AttributeError('unexpected arguments: {}'.format(kwargs))

        super().__init__()

        if type:
            self.type.extend(type)


class QueryResult(AQueryResult):

    values = EAttribute(eType=EJavaObject, unique=False, derived=False, changeable=True, upper=-1)

    def __init__(self, values=None, **kwargs):

        super().__init__(**kwargs)

        if values:
            self.values.extend(values)


class SerializableQueryResult(AQueryResult):

    values = EAttribute(eType=EString, unique=False, derived=False, changeable=True, upper=-1)

    def __init__(self, values=None, **kwargs):

        super().__init__(**kwargs)

        if values:
            self.values.extend(values)


class DataSource(Node):

    dataSourceService = EAttribute(eType=EString, derived=False, changeable=True)
    url = EAttribute(eType=EString, derived=False, changeable=True)
    libraryConfigurations = EReference(ordered=True, unique=True, containment=True, upper=-1)
    queries = EReference(ordered=True, unique=True, containment=True, upper=-1)
    dependenciesLibrary = EReference(ordered=True, unique=True, containment=False, upper=-1)
    targetLibrary = EReference(ordered=True, unique=True, containment=False)
    fetchVariableQuery = EReference(ordered=True, unique=True, containment=True)

    def __init__(self, dataSourceService=None, libraryConfigurations=None, url=None, queries=None, dependenciesLibrary=None, targetLibrary=None, fetchVariableQuery=None, **kwargs):

        super().__init__(**kwargs)

        if dataSourceService is not None:
            self.dataSourceService = dataSourceService

        if url is not None:
            self.url = url

        if libraryConfigurations:
            self.libraryConfigurations.extend(libraryConfigurations)

        if queries:
            self.queries.extend(queries)

        if dependenciesLibrary:
            self.dependenciesLibrary.extend(dependenciesLibrary)

        if targetLibrary is not None:
            self.targetLibrary = targetLibrary

        if fetchVariableQuery is not None:
            self.fetchVariableQuery = fetchVariableQuery


@abstract
class Query(Node):

    description = EAttribute(eType=EString, derived=False, changeable=True)
    runForCount = EAttribute(eType=EBoolean, derived=False, changeable=True, default_value=True)
    matchingCriteria = EReference(ordered=True, unique=True, containment=True, upper=-1)
    returnType = EReference(ordered=True, unique=True, containment=False)

    def __init__(self, description=None, matchingCriteria=None, runForCount=None, returnType=None, **kwargs):

        super().__init__(**kwargs)

        if description is not None:
            self.description = description

        if runForCount is not None:
            self.runForCount = runForCount

        if matchingCriteria:
            self.matchingCriteria.extend(matchingCriteria)

        if returnType is not None:
            self.returnType = returnType


class ProcessQuery(Query):

    queryProcessorId = EAttribute(eType=EString, derived=False, changeable=True)
    parameters = EReference(ordered=True, unique=True, containment=True, upper=-1)

    def __init__(self, parameters=None, queryProcessorId=None, **kwargs):

        super().__init__(**kwargs)

        if queryProcessorId is not None:
            self.queryProcessorId = queryProcessorId

        if parameters:
            self.parameters.extend(parameters)


class SimpleQuery(Query):

    query = EAttribute(eType=EString, derived=False, changeable=True)
    countQuery = EAttribute(eType=EString, derived=False, changeable=True)

    def __init__(self, query=None, countQuery=None, **kwargs):

        super().__init__(**kwargs)

        if query is not None:
            self.query = query

        if countQuery is not None:
            self.countQuery = countQuery


class CompoundQuery(Query):
    """Compound queries allow creating composite queries which chain together multiple queries. The results of a query in the chain will be fed to the subsequent query."""
    queryChain = EReference(ordered=True, unique=True, containment=True, upper=-1)

    def __init__(self, queryChain=None, **kwargs):

        super().__init__(**kwargs)

        if queryChain:
            self.queryChain.extend(queryChain)


class CompoundRefQuery(Query):
    """Compound ref queries make it possible to reference queries from any datasource"""
    queryChain = EReference(ordered=True, unique=True, containment=False, upper=-1)

    def __init__(self, queryChain=None, **kwargs):

        super().__init__(**kwargs)

        if queryChain:
            self.queryChain.extend(queryChain)



from pygeppetto.constants import ExperimentStatus

from .data_model import GeppettoProject


class LocalUser(object):
    def __init__(self, id, group, login=None, password=None, name=None,
                 projects=None):
        self.id = id
        self.login = login
        self.password = password
        self.name = name
        self.projects = []
        if projects:
            self.projects.extend(projects)
        self.group = group


class LocalGeppettoProject(GeppettoProject):
    pass


class LocalUserGroup(object):
    def __init__(self, name, privileges=None, space_allowance=None,
                 time_allowance=None):
        self.id = None
        self.name = name
        self.privileges = []
        if privileges:
            self.privileges.extend(privileges)
        self.space_allowance = space_allowance
        self.simulation_time_allowance = time_allowance


class LocalExperiment(object):
    def __init__(self, name, project, description=None,
                 status=ExperimentStatus.DESIGN, simulation_results=None,
                 script=None, view=None):
        self.id = None
        self.name = name
        self.parent_project = project
        self.description = description
        self.status = status
        self.simulation_results = []
        if simulation_results:
            self.simulation_results.extend(simulation_results)
        self.script = script
        self.view = view
        self.creation_date = None
        self.last_modified = None
        self.start_date = None
        self.last_ran = None



import sys

OutboundMessages = sys.modules[__name__]

LOAD_PROJECT = "load_project"

READ_URL_PARAMETERS = "read_url_parameters"

PROJECT_LOADED = "project_loaded"

ERROR_LOADING_PROJECT = "error_loading_project"

SERVER_UNAVAILABLE = "server_unavailable"

SERVER_AVAILABLE = "server_available"

EXPERIMENT_RUNNING = "experiment_running"

INFO_MESSAGE = "info_message"

SIMULATION_CONFIGURATION = "simulation_configuration"

ERROR = "generic_error"

ERROR_RUNNING_EXPERIMENT = "error_running_experiment"

ERROR_LOADING_SIMULATION_CONFIG = "error_loading_simulation_config"

ERROR_READING_SCRIPT = "error_reading_script"

ERROR_SETTING_WATCHED_VARIABLES = "error_setting_watched_variables"

ERROR_DOWNLOADING_MODEL = "error_downloading_model"

ERROR_DOWNLOADING_RESULTS = "error_downloading_results"

GEPPETTO_VERSION = "geppetto_version"

SCRIPT_FETCHED = "script_fetched"

DATASOURCE_RESULTS_FETCHED = "data_source_results_fetched"

GET_SCRIPTS = "get_scripts"

WATCHED_VARIABLES_SET = "watched_variables_set"

CLEAR_WATCH = "clear_watch"

CLIENT_ID = "client_id"

FIRE_SIM_SCRIPTS = "fire_sim_scripts"

SIMULATION_OVER = "simulation_over"

GET_SUPPORTED_OUTPUTS = "get_supported_outputs"

DOWNLOAD_MODEL = "download_model"

SET_PARAMETERS = "set_parameters"

NO_FEATURE = "no_feature"

EXPERIMENT_LOADING = "experiment_loading"

EXPERIMENT_LOADED = "experiment_loaded"

EXPERIMENT_STATUS = "experiment_status"

DELETE_EXPERIMENT = "experiment_deleted"

GET_EXPERIMENT_STATE = "get_experiment_state"

PROJECT_PERSISTED = "project_persisted"

DOWNLOAD_PROJECT = "download_project"

PROJECT_PROPS_SAVED = "project_props_saved"

PROJECT_MADE_PUBLIC = "project_made_public"

EXPERIMENT_PROPS_SAVED = "experiment_props_saved"

DROPBOX_LINKED = "dropbox_linked"

DROPBOX_UNLINKED = "dropbox_unlinked"

GET_DROPBOX_TOKEN = "get_dropbox_token"

RESULTS_UPLOADED = "results_uploaded"

MODEL_UPLOADED = "model_uploaded"

DOWNLOAD_RESULTS = "download_results"

UPDATE_MODEL_TREE = "update_model_tree"

EXPERIMENT_CREATED = "experiment_created"

EXPERIMENT_CLONED = "experiment_cloned"

EXPERIMENT_BATCH_CREATED = "experiment_batch_created"

GEPPETTO_MODEL_LOADED = "geppetto_model_loaded"

VARIABLE_FETCHED = "variable_fetched"

USER_PRIVILEGES = "user_privileges"

IMPORT_TYPE_RESOLVED = "import_type_resolved"

IMPORT_VALUE_RESOLVED = "import_value_resolved"

RETURN_QUERY = "return_query"

RETURN_QUERY_COUNT = "return_query_count"

RETURN_QUERY_RESULTS = "return_query_results"

FETCHED = 'fetched'



from pyecore.resources import global_registry
from .model import getEClassifier, eClassifiers
from .model import name, nsURI, nsPrefix, eClass
from .model import GeppettoModel, Node, GeppettoLibrary, LibraryManager, ExperimentState, VariableValue, Tag, DomainModel, ModelFormat, ExternalDomainModel, FileFormat, StringToStringMap, ISynchable, World

from .instances import Instance
from .variables import Variable
from .datasources import DataSource, Query
from .types import Type
from .values import Value, Pointer

from .types import PointType, QuantityType, CompositeType, ConnectionType, ArgumentType, URLType, PointerType, TextType, DynamicsType, SimpleArrayType, ImageType, JSONType, Type, VisualType, StateVariableType, HTMLType, CompositeVisualType, ParameterType, ArrayType, ExpressionType
from .values import VisualValue, ArrayElement, Unit, VisualGroupElement, Pointer, AArrayValue, FunctionPlot, Expression, Argument, GenericArray, VisualGroup, Point, PhysicalQuantity, PointerElement, JSON, HTML, SkeletonTransformation, Image, Function, Text, Cylinder, URL, TimeSeries, StringToValueMap, Value, ArrayValue, Metadata, Particles, SkeletonAnimation, Connection, Composite, MDTimeSeries, Quantity, Dynamics
from .variables import Variable, TypeToValueMap
from .datasources import CompoundQuery, AQueryResult, QueryResults, Query, ProcessQuery, CompoundRefQuery, DataSource, DataSourceLibraryConfiguration, QueryMatchingCriteria
from .instances import SimpleConnectionInstance, Instance, SimpleInstance
from . import model
from . import types

from . import values

from . import variables

from . import datasources

from . import instances


__all__ = ['GeppettoModel', 'Node', 'GeppettoLibrary', 'LibraryManager', 'ExperimentState', 'VariableValue', 'Tag',
           'DomainModel', 'ModelFormat', 'ExternalDomainModel', 'FileFormat', 'StringToStringMap', 'ISynchable', 'World']

eSubpackages = [types, values, variables, datasources, instances]
eSuperPackage = None
model.eSubpackages = eSubpackages
model.eSuperPackage = eSuperPackage

Type.superType.eType = Type
Type.visualType.eType = VisualType
Type.domainModel.eType = DomainModel
VisualType.defaultValue.eType = VisualValue
CompositeType.variables.eType = Variable
CompositeType.defaultValue.eType = Composite
PointerType.defaultValue.eType = Pointer
QuantityType.defaultValue.eType = Quantity
ParameterType.defaultValue.eType = Quantity
StateVariableType.defaultValue.eType = Quantity
DynamicsType.defaultValue.eType = Dynamics
ArgumentType.defaultValue.eType = Argument
ExpressionType.defaultValue.eType = Expression
HTMLType.defaultValue.eType = HTML
JSONType.defaultValue.eType = JSON
TextType.defaultValue.eType = Text
URLType.defaultValue.eType = URL
PointType.defaultValue.eType = Point
ArrayType.arrayType.eType = Type
ArrayType.defaultValue.eType = ArrayValue
CompositeVisualType.variables.eType = Variable
CompositeVisualType.visualGroups.eType = VisualGroup
ConnectionType.variables.eType = Variable
ConnectionType.defaultValue.eType = Composite
ImageType.defaultValue.eType = Image
SimpleArrayType.defaultValue.eType = AArrayValue
Composite.value.eType = StringToValueMap
StringToValueMap.value.eType = Value
PhysicalQuantity.unit.eType = Unit
TimeSeries.unit.eType = Unit
MDTimeSeries.value.eType = Value
Pointer.elements.eType = PointerElement
Pointer.point.eType = Point
PointerElement.variable.eType = Variable
PointerElement.type.eType = Type
Dynamics.initialCondition.eType = PhysicalQuantity
Dynamics.dynamics.eType = Function
Function.arguments.eType = Argument
Function.expression.eType = Expression
Function.functionPlot.eType = FunctionPlot
VisualValue.groupElements.eType = VisualGroupElement
VisualValue.position.eType = Point
Cylinder.distal.eType = Point
Particles.particles.eType = Point
SkeletonAnimation.skeletonTransformationSeries.eType = SkeletonTransformation
VisualGroupElement.parameter.eType = Quantity
VisualGroup.visualGroupElements.eType = VisualGroupElement
Connection.a.eType = Pointer
Connection.b.eType = Pointer
ArrayElement.position.eType = Point
ArrayElement.initialValue.eType = Value
ArrayValue.elements.eType = ArrayElement
Metadata.value.eType = StringToValueMap
GenericArray.elements.eType = Value
Variable.anonymousTypes.eType = Type
Variable.initialValues.eType = TypeToValueMap
Variable.position.eType = Point
TypeToValueMap.key.eType = Type
TypeToValueMap.value.eType = Value
DataSource.libraryConfigurations.eType = DataSourceLibraryConfiguration
DataSource.queries.eType = Query
DataSource.dependenciesLibrary.eType = GeppettoLibrary
DataSource.targetLibrary.eType = GeppettoLibrary
DataSource.fetchVariableQuery.eType = Query
DataSourceLibraryConfiguration.library.eType = GeppettoLibrary
Query.matchingCriteria.eType = QueryMatchingCriteria
Query.returnType.eType = Type
ProcessQuery.parameters.eType = StringToStringMap
CompoundQuery.queryChain.eType = Query
CompoundRefQuery.queryChain.eType = Query
QueryResults.results.eType = AQueryResult
QueryMatchingCriteria.type.eType = Type
Instance.type.eType = Type
Instance.value.eType = Value
SimpleInstance.visualValue.eType = VisualValue
SimpleInstance.position.eType = Point
SimpleConnectionInstance.a.eType = Instance
SimpleConnectionInstance.b.eType = Instance
GeppettoModel.variables.eType = Variable
GeppettoModel.worlds.eType = World
GeppettoModel.libraries.eType = GeppettoLibrary
GeppettoModel.tags.eType = Tag
GeppettoModel.dataSources.eType = DataSource
GeppettoModel.queries.eType = Query
Node.tags.eType = Tag
GeppettoLibrary.types.eType = Type
GeppettoLibrary.sharedTypes.eType = Type
LibraryManager.libraries.eType = GeppettoLibrary
ExperimentState.recordedVariables.eType = VariableValue
ExperimentState.setParameters.eType = VariableValue
VariableValue.pointer.eType = Pointer
VariableValue.value.eType = Value
Tag.tags.eType = Tag
DomainModel.format.eType = ModelFormat
World.variables.eType = Variable
World.instances.eType = Instance
Type.referencedVariables.eType = Variable
Variable.types.eType = Type
Variable.types.eOpposite = Type.referencedVariables

otherClassifiers = [FileFormat]

for classif in otherClassifiers:
    eClassifiers[classif.name] = classif
    classif.ePackage = eClass

for classif in eClassifiers.values():
    eClass.eClassifiers.append(classif.eClass)

for subpack in eSubpackages:
    eClass.eSubpackages.append(subpack.eClass)

register_packages = [model] + eSubpackages
for pack in register_packages:
    global_registry[pack.nsURI] = pack



"""Definition of meta model 'values'."""
from functools import partial
import pyecore.ecore as Ecore
from pyecore.ecore import *
from ..model import Node, ISynchable


name = 'values'
nsURI = 'https://raw.githubusercontent.com/openworm/org.geppetto.model/master/src/main/resources/geppettoModel.ecore#//values'
nsPrefix = 'gep'

eClass = EPackage(name=name, nsURI=nsURI, nsPrefix=nsPrefix)

eClassifiers = {}
getEClassifier = partial(Ecore.getEClassifier, searchspace=eClassifiers)
Connectivity = EEnum('Connectivity', literals=['DIRECTIONAL', 'BIDIRECTIONAL', 'NON_DIRECTIONAL'])

ImageFormat = EEnum('ImageFormat', literals=[
                    'PNG', 'JPEG', 'IIP', 'DCM', 'NIFTI', 'TIFF', 'DZI', 'GOOGLE_MAP'])


class StringToValueMap(EObject, metaclass=MetaEClass):

    key = EAttribute(eType=EString, derived=False, changeable=True)
    value = EReference(ordered=True, unique=True, containment=True)

    def __init__(self, key=None, value=None, **kwargs):
        if kwargs:
            raise AttributeError('unexpected arguments: {}'.format(kwargs))

        super().__init__()

        if key is not None:
            self.key = key

        if value is not None:
            self.value = value


class PointerElement(EObject, metaclass=MetaEClass):

    index = EAttribute(eType=EInt, derived=False, changeable=True, default_value=-1)
    variable = EReference(ordered=True, unique=True, containment=False)
    type = EReference(ordered=True, unique=True, containment=False)

    def __init__(self, variable=None, type=None, index=None, **kwargs):
        if kwargs:
            raise AttributeError('unexpected arguments: {}'.format(kwargs))

        super().__init__()

        if index is not None:
            self.index = index

        if variable is not None:
            self.variable = variable

        if type is not None:
            self.type = type


class FunctionPlot(EObject, metaclass=MetaEClass):

    title = EAttribute(eType=EString, derived=False, changeable=True)
    xAxisLabel = EAttribute(eType=EString, derived=False, changeable=True)
    yAxisLabel = EAttribute(eType=EString, derived=False, changeable=True)
    initialValue = EAttribute(eType=EDouble, derived=False, changeable=True)
    finalValue = EAttribute(eType=EDouble, derived=False, changeable=True)
    stepValue = EAttribute(eType=EDouble, derived=False, changeable=True)

    def __init__(self, title=None, xAxisLabel=None, yAxisLabel=None, initialValue=None, finalValue=None, stepValue=None, **kwargs):
        if kwargs:
            raise AttributeError('unexpected arguments: {}'.format(kwargs))

        super().__init__()

        if title is not None:
            self.title = title

        if xAxisLabel is not None:
            self.xAxisLabel = xAxisLabel

        if yAxisLabel is not None:
            self.yAxisLabel = yAxisLabel

        if initialValue is not None:
            self.initialValue = initialValue

        if finalValue is not None:
            self.finalValue = finalValue

        if stepValue is not None:
            self.stepValue = stepValue


class SkeletonTransformation(EObject, metaclass=MetaEClass):

    skeletonTransformation = EAttribute(eType=EDouble, derived=False, changeable=True, upper=-1)

    def __init__(self, skeletonTransformation=None, **kwargs):
        if kwargs:
            raise AttributeError('unexpected arguments: {}'.format(kwargs))

        super().__init__()

        if skeletonTransformation:
            self.skeletonTransformation.extend(skeletonTransformation)


@abstract
class Value(ISynchable):

    def __init__(self, **kwargs):

        super().__init__(**kwargs)


class Composite(Value):

    value = EReference(ordered=True, unique=True, containment=True, upper=-1)

    def __init__(self, value=None, **kwargs):

        super().__init__(**kwargs)

        if value:
            self.value.extend(value)


class Quantity(Value):

    scalingFactor = EAttribute(eType=EInt, derived=False, changeable=True)
    value = EAttribute(eType=EDouble, derived=False, changeable=True)

    def __init__(self, scalingFactor=None, value=None, **kwargs):

        super().__init__(**kwargs)

        if scalingFactor is not None:
            self.scalingFactor = scalingFactor

        if value is not None:
            self.value = value


class Unit(Value):

    unit = EAttribute(eType=EString, derived=False, changeable=True)

    def __init__(self, unit=None, **kwargs):

        super().__init__(**kwargs)

        if unit is not None:
            self.unit = unit


class TimeSeries(Value):

    scalingFactor = EAttribute(eType=EInt, derived=False, changeable=True)
    value = EAttribute(eType=EDouble, unique=False, derived=False, changeable=True, upper=-1)
    unit = EReference(ordered=True, unique=True, containment=True)

    def __init__(self, unit=None, scalingFactor=None, value=None, **kwargs):

        super().__init__(**kwargs)

        if scalingFactor is not None:
            self.scalingFactor = scalingFactor

        if value:
            self.value.extend(value)

        if unit is not None:
            self.unit = unit


class MDTimeSeries(Value):

    value = EReference(ordered=True, unique=True, containment=True, upper=-1)

    def __init__(self, value=None, **kwargs):

        super().__init__(**kwargs)

        if value:
            self.value.extend(value)


@abstract
class MetadataValue(Value):

    def __init__(self, **kwargs):

        super().__init__(**kwargs)


class Pointer(Value):

    path = EAttribute(eType=EString, derived=False, changeable=True)
    elements = EReference(ordered=True, unique=True, containment=True, upper=-1)
    point = EReference(ordered=True, unique=True, containment=True)

    def __init__(self, elements=None, point=None, path=None, **kwargs):

        super().__init__(**kwargs)

        if path is not None:
            self.path = path

        if elements:
            self.elements.extend(elements)

        if point is not None:
            self.point = point

    def getInstancePath(self):

        raise NotImplementedError('operation getInstancePath(...) not yet implemented')


class Point(Value):

    x = EAttribute(eType=EDouble, derived=False, changeable=True)
    y = EAttribute(eType=EDouble, derived=False, changeable=True)
    z = EAttribute(eType=EDouble, derived=False, changeable=True)

    def __init__(self, x=None, y=None, z=None, **kwargs):

        super().__init__(**kwargs)

        if x is not None:
            self.x = x

        if y is not None:
            self.y = y

        if z is not None:
            self.z = z


class Dynamics(Value):

    initialCondition = EReference(ordered=True, unique=True, containment=True)
    dynamics = EReference(ordered=True, unique=True, containment=True)

    def __init__(self, initialCondition=None, dynamics=None, **kwargs):

        super().__init__(**kwargs)

        if initialCondition is not None:
            self.initialCondition = initialCondition

        if dynamics is not None:
            self.dynamics = dynamics


class Function(Value):

    arguments = EReference(ordered=True, unique=True, containment=True, upper=-1)
    expression = EReference(ordered=True, unique=True, containment=True)
    functionPlot = EReference(ordered=True, unique=True, containment=True)

    def __init__(self, arguments=None, expression=None, functionPlot=None, **kwargs):

        super().__init__(**kwargs)

        if arguments:
            self.arguments.extend(arguments)

        if expression is not None:
            self.expression = expression

        if functionPlot is not None:
            self.functionPlot = functionPlot


class Argument(Value):

    argument = EAttribute(eType=EString, derived=False, changeable=True)

    def __init__(self, argument=None, **kwargs):

        super().__init__(**kwargs)

        if argument is not None:
            self.argument = argument


class Expression(Value):

    expression = EAttribute(eType=EString, derived=False, changeable=True)

    def __init__(self, expression=None, **kwargs):

        super().__init__(**kwargs)

        if expression is not None:
            self.expression = expression


@abstract
class VisualValue(Value):

    groupElements = EReference(ordered=True, unique=True, containment=False, upper=-1)
    position = EReference(ordered=True, unique=True, containment=True)

    def __init__(self, groupElements=None, position=None, **kwargs):

        super().__init__(**kwargs)

        if groupElements:
            self.groupElements.extend(groupElements)

        if position is not None:
            self.position = position


class Particles(Value):

    particles = EReference(ordered=True, unique=True, containment=True, upper=-1)

    def __init__(self, particles=None, **kwargs):

        super().__init__(**kwargs)

        if particles:
            self.particles.extend(particles)


class VisualGroupElement(Node):

    defaultColor = EAttribute(eType=EString, derived=False, changeable=True)
    parameter = EReference(ordered=True, unique=True, containment=True)

    def __init__(self, defaultColor=None, parameter=None, **kwargs):

        super().__init__(**kwargs)

        if defaultColor is not None:
            self.defaultColor = defaultColor

        if parameter is not None:
            self.parameter = parameter


class VisualGroup(Node):

    lowSpectrumColor = EAttribute(eType=EString, derived=False, changeable=True)
    highSpectrumColor = EAttribute(eType=EString, derived=False, changeable=True)
    type = EAttribute(eType=EString, derived=False, changeable=True)
    visualGroupElements = EReference(ordered=True, unique=True, containment=True, upper=-1)

    def __init__(self, lowSpectrumColor=None, highSpectrumColor=None, type=None, visualGroupElements=None, **kwargs):

        super().__init__(**kwargs)

        if lowSpectrumColor is not None:
            self.lowSpectrumColor = lowSpectrumColor

        if highSpectrumColor is not None:
            self.highSpectrumColor = highSpectrumColor

        if type is not None:
            self.type = type

        if visualGroupElements:
            self.visualGroupElements.extend(visualGroupElements)


class Connection(Value):

    connectivity = EAttribute(eType=Connectivity, derived=False, changeable=True)
    a = EReference(ordered=True, unique=True, containment=True)
    b = EReference(ordered=True, unique=True, containment=True)

    def __init__(self, a=None, b=None, connectivity=None, **kwargs):

        super().__init__(**kwargs)

        if connectivity is not None:
            self.connectivity = connectivity

        if a is not None:
            self.a = a

        if b is not None:
            self.b = b


class ArrayElement(Value):

    index = EAttribute(eType=EInt, derived=False, changeable=True)
    position = EReference(ordered=True, unique=True, containment=True)
    initialValue = EReference(ordered=True, unique=True, containment=True)

    def __init__(self, index=None, position=None, initialValue=None, **kwargs):

        super().__init__(**kwargs)

        if index is not None:
            self.index = index

        if position is not None:
            self.position = position

        if initialValue is not None:
            self.initialValue = initialValue


class ArrayValue(Value):

    elements = EReference(ordered=True, unique=True, containment=True, upper=-1)

    def __init__(self, elements=None, **kwargs):

        super().__init__(**kwargs)

        if elements:
            self.elements.extend(elements)


class Image(Value):

    data = EAttribute(eType=EString, derived=False, changeable=True)
    name = EAttribute(eType=EString, derived=False, changeable=True)
    reference = EAttribute(eType=EString, derived=False, changeable=True)
    format = EAttribute(eType=ImageFormat, derived=False, changeable=True)

    def __init__(self, data=None, name=None, reference=None, format=None, **kwargs):

        super().__init__(**kwargs)

        if data is not None:
            self.data = data

        if name is not None:
            self.name = name

        if reference is not None:
            self.reference = reference

        if format is not None:
            self.format = format


class ImportValue(Value):

    modelInterpreterId = EAttribute(eType=EString, derived=False, changeable=True)

    def __init__(self, modelInterpreterId=None, **kwargs):

        super().__init__(**kwargs)

        if modelInterpreterId is not None:
            self.modelInterpreterId = modelInterpreterId


@abstract
class AArrayValue(Value):

    def __init__(self, **kwargs):

        super().__init__(**kwargs)


class PhysicalQuantity(Quantity):

    unit = EReference(ordered=True, unique=True, containment=True)

    def __init__(self, unit=None, **kwargs):

        super().__init__(**kwargs)

        if unit is not None:
            self.unit = unit


class Text(MetadataValue):

    text = EAttribute(eType=EString, derived=False, changeable=True)

    def __init__(self, text=None, **kwargs):

        super().__init__(**kwargs)

        if text is not None:
            self.text = text


class URL(MetadataValue):

    url = EAttribute(eType=EString, derived=False, changeable=True)

    def __init__(self, url=None, **kwargs):

        super().__init__(**kwargs)

        if url is not None:
            self.url = url


class HTML(MetadataValue):

    html = EAttribute(eType=EString, derived=False, changeable=True)

    def __init__(self, html=None, **kwargs):

        super().__init__(**kwargs)

        if html is not None:
            self.html = html


class Collada(VisualValue):

    collada = EAttribute(eType=EString, derived=False, changeable=True)

    def __init__(self, collada=None, **kwargs):

        super().__init__(**kwargs)

        if collada is not None:
            self.collada = collada


class OBJ(VisualValue):

    obj = EAttribute(eType=EString, derived=False, changeable=True)

    def __init__(self, obj=None, **kwargs):

        super().__init__(**kwargs)

        if obj is not None:
            self.obj = obj


class Sphere(VisualValue):

    radius = EAttribute(eType=EDouble, derived=False, changeable=True)

    def __init__(self, radius=None, **kwargs):

        super().__init__(**kwargs)

        if radius is not None:
            self.radius = radius


class Cylinder(VisualValue):

    bottomRadius = EAttribute(eType=EDouble, derived=False, changeable=True)
    topRadius = EAttribute(eType=EDouble, derived=False, changeable=True)
    height = EAttribute(eType=EDouble, derived=False, changeable=True)
    distal = EReference(ordered=True, unique=True, containment=True)

    def __init__(self, bottomRadius=None, topRadius=None, height=None, distal=None, **kwargs):

        super().__init__(**kwargs)

        if bottomRadius is not None:
            self.bottomRadius = bottomRadius

        if topRadius is not None:
            self.topRadius = topRadius

        if height is not None:
            self.height = height

        if distal is not None:
            self.distal = distal


class SkeletonAnimation(VisualValue):

    skeletonTransformationSeries = EReference(
        ordered=True, unique=True, containment=False, upper=-1)

    def __init__(self, skeletonTransformationSeries=None, **kwargs):

        super().__init__(**kwargs)

        if skeletonTransformationSeries:
            self.skeletonTransformationSeries.extend(skeletonTransformationSeries)


class Metadata(MetadataValue):

    value = EReference(ordered=True, unique=True, containment=True, upper=-1)

    def __init__(self, value=None, **kwargs):

        super().__init__(**kwargs)

        if value:
            self.value.extend(value)


class JSON(MetadataValue):

    json = EAttribute(eType=EString, derived=False, changeable=True)

    def __init__(self, json=None, **kwargs):

        super().__init__(**kwargs)

        if json is not None:
            self.json = json


class GenericArray(AArrayValue):

    elements = EReference(ordered=True, unique=True, containment=True, upper=-1)

    def __init__(self, elements=None, **kwargs):

        super().__init__(**kwargs)

        if elements:
            self.elements.extend(elements)


class StringArray(AArrayValue):

    elements = EAttribute(eType=EString, unique=False, derived=False, changeable=True, upper=-1)

    def __init__(self, elements=None, **kwargs):

        super().__init__(**kwargs)

        if elements:
            self.elements.extend(elements)


class IntArray(AArrayValue):

    elements = EAttribute(eType=EInt, unique=False, derived=False, changeable=True, upper=-1)

    def __init__(self, elements=None, **kwargs):

        super().__init__(**kwargs)

        if elements:
            self.elements.extend(elements)


class DoubleArray(AArrayValue):

    elements = EAttribute(eType=EDouble, unique=False, derived=False, changeable=True, upper=-1)

    def __init__(self, elements=None, **kwargs):

        super().__init__(**kwargs)

        if elements:
            self.elements.extend(elements)



#!/usr/bin/env python

from setuptools import setup, find_packages

__version__ = "0.7.0"

required_packages = [
    'pyecore==0.11.0',
    'multimethod',
    'deprecated',
    'airspeed',
    'requests>=2.0.0'
]


setup(
    name="pygeppetto",
    version=__version__,
    description=("Geppetto Python API for creating a Geppetto Model from Python"),
    author="The Geppetto Development Team",
    author_email="info@geppetto.org",
    license="MIT",
    long_description=open('README.rst').read(),
    keywords="geppetto openworm Python",
    url="https://github.com/openworm/pygeppetto",
    packages=find_packages(),
    package_data={'': ['README.md']},
    include_package_data=True,
    install_requires=required_packages,
    extras_require={'testing': ['pytest', 'responses']},
    classifiers=[
        "Programming Language :: Python",
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        "Operating System :: OS Independent",
        "Intended Audience :: Developers",
        "Topic :: Software Development",
        "Topic :: Software Development :: Libraries"
    ]
)



import json

from .outbound_messages import *


class Resources:
    ERROR_LOADING_PROJECT_MESSAGE = "Invalid project file. Double check the information you have entered and try again."

    ERROR_DOWNLOADING_MODEL = "Format not supported"

    UNSUPPORTED_OPERATION = "This deployment of Geppetto doesn't support this operation. Contact info@geppetto.org for more information."

    VOLATILE_PROJECT = "The operation cannot be executed on a volatile project. If you wish to persist the project press the star icon at the top."


class GeppettoTransportMessage:

    def __init__(self, requestID, type_, data):
        self.requestID = requestID
        self.type = type_
        self.data = data


class TransportMessageFactory(object):
    """ generated source for class TransportMessageFactory """
    EMPTY_STRING = ""

    #
    # 	 * Create JSON object with appropriate message for its type
    # 	 * @param id
    # 	 *
    # 	 * @param type - Type of message of requested
    # 	 * @return
    #
    @classmethod
    def getTransportMessage(cls, requestID, type_, update):
        """ generated source for method getTransportMessage """
        messageType = type_

        payload = update if type(update) == dict else {}
        if type_ == ERROR:
            if (type(update) == dict):
                payload.update(update)
            else:
                payload.update({'message': update})
        elif type_ == INFO_MESSAGE:
            payload.update({'message': update})
        elif type_ == ERROR_LOADING_PROJECT:
            payload.update({"message": Resources.ERROR_LOADING_PROJECT_MESSAGE})
        elif type_ == ERROR_DOWNLOADING_MODEL:
            payload.update({"message": Resources.ERROR_DOWNLOADING_MODEL})
        elif type_ == ERROR_DOWNLOADING_RESULTS:
            payload.update({"message": Resources.ERROR_DOWNLOADING_MODEL})
        elif type_ == READ_URL_PARAMETERS:
            pass
        elif type_ == USER_PRIVILEGES:
            payload.update(
                {OutboundMessages.USER_PRIVILEGES: update if (update != None) else cls.EMPTY_STRING})
        elif type_ == PROJECT_LOADED:
            payload.update({OutboundMessages.PROJECT_LOADED: update if (update != None) else cls.EMPTY_STRING})
        elif type_ == GEPPETTO_MODEL_LOADED:
            payload.update(
                {OutboundMessages.GEPPETTO_MODEL_LOADED: update if (update != None) else cls.EMPTY_STRING})
        elif type_ == VARIABLE_FETCHED:
            payload.update(
                {OutboundMessages.VARIABLE_FETCHED: update if (update != None) else cls.EMPTY_STRING})
        elif type_ == FETCHED:
            payload.update(
                {OutboundMessages.FETCHED: update if (update != None) else cls.EMPTY_STRING})
        elif type_ == IMPORT_TYPE_RESOLVED:
            payload.update(
                {OutboundMessages.IMPORT_TYPE_RESOLVED: update if (update != None) else cls.EMPTY_STRING})
        elif type_ == SIMULATION_OVER:
            payload.update(
                {OutboundMessages.SIMULATION_OVER: update if (update != None) else cls.EMPTY_STRING})
        elif type_ == FIRE_SIM_SCRIPTS:
            payload.update({OutboundMessages.GET_SCRIPTS: update if (update != None) else cls.EMPTY_STRING})
        elif type_ == EXPERIMENT_RUNNING:
            payload.update({"update": update if (update != None) else cls.EMPTY_STRING})
        elif type_ == EXPERIMENT_STATUS:
            payload.update({"update": update if (update != None) else cls.EMPTY_STRING})
        elif type_ == DOWNLOAD_MODEL:
            payload.update({"update": update if (update != None) else cls.EMPTY_STRING})
        elif type_ == DOWNLOAD_PROJECT:
            payload.update({"update": update if (update != None) else cls.EMPTY_STRING})
        elif type_ == GET_EXPERIMENT_STATE:
            payload.update({"update": update if (update != None) else cls.EMPTY_STRING})
        elif type_ == DELETE_EXPERIMENT:
            payload.update({"update": update if (update != None) else cls.EMPTY_STRING})
        elif type_ == SIMULATION_CONFIGURATION:
            payload.update({"configuration": update if (update != None) else cls.EMPTY_STRING})
        elif type_ == CLIENT_ID:
            payload.update({"clientID": update if (update != None) else cls.EMPTY_STRING})
        elif type_ == PROJECT_PERSISTED:
            payload.update({"update": update if (update != None) else cls.EMPTY_STRING})
        elif type_ == PROJECT_MADE_PUBLIC:
            payload.update({"update": update if (update != None) else cls.EMPTY_STRING})
        elif type_ == PROJECT_PROPS_SAVED:
            payload.update({"update": update if (update != None) else cls.EMPTY_STRING})
        elif type_ == EXPERIMENT_PROPS_SAVED:
            payload.update({"update": update if (update != None) else cls.EMPTY_STRING})
        elif type_ == DROPBOX_LINKED:
            payload.update({OutboundMessages.DROPBOX_LINKED, update if (update != None) else cls.EMPTY_STRING})
        elif type_ == DROPBOX_UNLINKED:
            payload.update(
                {OutboundMessages.DROPBOX_UNLINKED, update if (update != None) else cls.EMPTY_STRING})
        elif type_ == RESULTS_UPLOADED:
            payload.update(
                {OutboundMessages.RESULTS_UPLOADED, update if (update != None) else cls.EMPTY_STRING})
        elif type_ == MODEL_UPLOADED:
            payload.update({OutboundMessages.MODEL_UPLOADED, update if (update != None) else cls.EMPTY_STRING})
        else:
            payload.update({type_: update if (update != None) else cls.EMPTY_STRING})
        return cls.createTransportMessage(requestID, messageType, json.dumps(payload))

    #
    # 	 * Create JSON object with type and parameters
    # 	 *
    # 	 * @param type - Type of message
    # 	 * @param params - list of name-value pairs representing parameter names and values
    # 	 * @return
    #
    @classmethod
    def createTransportMessage(cls, requestID, type_, data):
        return GeppettoTransportMessage(requestID=requestID, type_=type_, data=data)



import os

import pytest
from pyecore.resources import ResourceSet, URI
from pygeppetto.model.utils.pointer_utility import PointerUtility, GeppettoModelException


@pytest.fixture
def geppetto_model():
    res_set = ResourceSet()
    resource = res_set.get_resource(URI(os.path.join(os.path.dirname(__file__), "xmi-data/GeppettoModelTest2.xmi")))
    return resource.contents[0]


def test_get_pointer(geppetto_model):
    PointerUtility.get_pointer(geppetto_model, "addressBook[30].address.zone[4].area")
    PointerUtility.get_pointer(geppetto_model, "sample.person.name")


def test_equals(geppetto_model):
    p1 = PointerUtility.get_pointer(geppetto_model, "addressBook(addressBook)[3].name(genericParameter)")
    p2 = PointerUtility.get_pointer(geppetto_model, "addressBook(addressBook)[3].name(genericParameter)")
    assert p1 is not p2
    assert "addressBook(addressBook)[3].name(genericParameter)" == p1.get_instance_path()
    assert p1.get_instance_path() == p2.get_instance_path()
    assert PointerUtility.equals(p1, p2)
    p3 = PointerUtility.get_pointer(geppetto_model, "addressBook(addressBook)[3]")
    assert p1 is not p3
    assert not PointerUtility.equals(p1, p3)
    p4 = PointerUtility.get_pointer(geppetto_model, "addressBook(addressBook)[3].address(address)")
    assert p1 is not p3
    assert not (PointerUtility.equals(p1, p4))
    assert not (PointerUtility.equals(p3, p4))


def test_get_pointer_negative1(geppetto_model):
    with pytest.raises(GeppettoModelException):
        PointerUtility.get_pointer(geppetto_model, "addressBook(addressBok)[3].name(genericParameter)")


def test_get_pointer_negative2(geppetto_model):
    with pytest.raises(GeppettoModelException):
        PointerUtility.get_pointer(geppetto_model, "addresBook")


def test_get_pointer_negative3(geppetto_model):
    with pytest.raises(GeppettoModelException):
        PointerUtility.get_pointer(geppetto_model, "addressBook6]")


def test_get_pointer_negative4(geppetto_model):
    with pytest.raises(GeppettoModelException):
        PointerUtility.get_pointer(geppetto_model,
                                   "addressBook(addressBook)[30]address(address).zone(zone)[4].area(genericParameter)")


def test_get_pointer_negative5(geppetto_model):
    with pytest.raises(Exception):
        PointerUtility.get_pointer(geppetto_model, "addressBook[30].address.zone[].area")


def test_get_pointer_negative6(geppetto_model):
    with pytest.raises(GeppettoModelException):
        PointerUtility.get_pointer(geppetto_model, "addressBook[30].address.zone[2].arrea")


def test_get_pointer_negative7(geppetto_model):
    with pytest.raises(GeppettoModelException):
        PointerUtility.get_pointer(geppetto_model, "addressBook(addressBok)[3].")


def test_get_variable(geppetto_model):
    p = PointerUtility.get_pointer(geppetto_model, "addressBook(addressBook)[3].name(genericParameter)")
    assert "name" == PointerUtility.get_variable(p).id
    assert "name" == PointerUtility.get_variable(p).name
    assert "genericParameter" == PointerUtility.get_variable(p).types[0].id
    p = PointerUtility.get_pointer(geppetto_model, "addressBook")
    assert "addressBook" == PointerUtility.get_variable(p).id
    assert "addressBook" == PointerUtility.get_variable(p).name
    assert "addressBook" == PointerUtility.get_variable(p).types[0].id
    p = PointerUtility.get_pointer(geppetto_model, "addressBook[6]")
    assert "addressBook" == PointerUtility.get_variable(p).id
    assert "addressBook" == PointerUtility.get_variable(p).name
    assert "addressBook" == PointerUtility.get_variable(p).types[0].id
    assert 6 == p.elements[0].index
    p = PointerUtility.get_pointer(geppetto_model,
                                   "addressBook(addressBook)[30].address(address).zone(zone)[4].area(genericParameter)")
    assert "area", PointerUtility.get_variable(p).id
    assert "area", PointerUtility.get_variable(p).name
    assert "genericParameter" == PointerUtility.get_variable(p).types[0].id
    assert 30, p.elements[0].index
    assert -1, p.elements[1].index
    assert 4, p.elements[2].index
    p = PointerUtility.get_pointer(geppetto_model, "addressBook[30].address.zone[4].area")
    assert "area", PointerUtility.get_variable(p).id
    assert "area", PointerUtility.get_variable(p).name
    assert "genericParameter" == PointerUtility.get_variable(p).types[0].id
    assert 30, p.elements[0].index
    assert -1, p.elements[1].index
    assert 4, p.elements[2].index


def test_get_type(geppetto_model):
    """ generated source for method testget_type """
    p = PointerUtility.get_pointer(geppetto_model, "addressBook(addressBook)[3].name(genericParameter)")
    assert "genericParameter" == PointerUtility.get_type(p).id
    assert "genericParameter" == PointerUtility.get_type(p).name
    p = PointerUtility.get_pointer(geppetto_model, "addressBook")
    assert "addressBook" == PointerUtility.get_type(p).id
    assert "addressBook" == PointerUtility.get_type(p).name
    p = PointerUtility.get_pointer(geppetto_model, "addressBook[6]")
    assert "addressBook" == PointerUtility.get_type(p).id
    assert "addressBook" == PointerUtility.get_type(p).name
    assert int(6), p.elements[0].index
    p = PointerUtility.get_pointer(geppetto_model,
                                   "addressBook(addressBook)[30].address(address).zone(zone)[4].area(genericParameter)")
    assert "genericParameter" == PointerUtility.get_type(p).id
    assert "genericParameter" == PointerUtility.get_type(p).name
    p = PointerUtility.get_pointer(geppetto_model, "addressBook[30].address.zone[4].area")
    assert "genericParameter" == PointerUtility.get_type(p).id
    assert "genericParameter" == PointerUtility.get_type(p).name


def test_get_geppetto_library(geppetto_model):
    p = PointerUtility.get_pointer(geppetto_model, "addressBook(addressBook)[3].name(genericParameter)")
    assert "sample" == PointerUtility.get_geppetto_library(p).id
    assert "sample" == PointerUtility.get_geppetto_library(p).id
    p = PointerUtility.get_pointer(geppetto_model, "addressBook[6]")
    assert "sample" == PointerUtility.get_geppetto_library(p).id
    p = PointerUtility.get_pointer(geppetto_model,
                                   "addressBook(addressBook)[30].address(address).zone(zone)[4].area(genericParameter)")
    assert "sample" == PointerUtility.get_geppetto_library(p).id
    p = PointerUtility.get_pointer(geppetto_model, "addressBook[30].address.zone[4].area")
    assert "sample" == PointerUtility.get_geppetto_library(p).id


def test_get_instance_path(geppetto_model):
    p = PointerUtility.get_pointer(geppetto_model, "addressBook")
    assert "addressBook(addressBook)" == PointerUtility.get_instance_path(PointerUtility.get_variable(p),
                                                                          PointerUtility.get_type(p))



import os.path

class GeppettoManager():

    def __init__(self):
        pass

    def loadProject(self, project):
        pass

    def closeProject(self, project):
        pass

    def deleteProject(self, project):
        pass

    def persistProject(self, project):
        pass

    def checkExperimentsStatus(self, project):
        pass
	
    def makeProjectPublic(self, project, isPublic):
        pass

    def loadExperiment(self, experiment):
        pass

    def newExperiment(self, project):
        pass

    def runExperiment(self, experiment):
        pass
        
    def getExperimentState(self, experiment, filters):
        pass

    def deleteExperiment(self, experiment):
        pass
    
    def cancelExperimentRun(self, experiment):
        pass

    def setExperimentView(self, experiment, project):
        pass
    
    def cloneExperiment(self, project, originalExperiment):
        pass

    def setModelParameters(self, parameters, experiment, project):
        pass

    def setWatchedVariables(self, watchedVariables, experiment, project, watch):
        pass
	
    def resolveImportType(self, typePaths, geppettoProject):
        pass
	
    def resolveImportValue(self, path, experiment, geppettoProject):
        pass


	


from pygeppetto.model.model_access import GeppettoModelAccess
from pygeppetto.model.values import ImportValue


class ModelInterpreter():

    def importType(self, url, typeName, library, commonLibraryAccess: GeppettoModelAccess):
        raise NotImplementedError(
            "{} not implemented in class {}".format(ModelInterpreter.importType.__name__, self.__class__.__name__))

    def importValue(self, importValue: ImportValue):
        raise NotImplementedError(
            "{} not implemented in class {}".format(ModelInterpreter.importValue.__name__, self.__class__.__name__))

    def downloadModel(self, pointer, format, aspectConfiguration):
        raise NotImplementedError(
            "{} not implemented in class {}".format(ModelInterpreter.downloadModel.__name__, self.__class__.__name__))

    def getSupportedOutputs(self, pointer):
        raise NotImplementedError(
            "{} not implemented in class {}".format(ModelInterpreter.getSupportedOutputs.__name__,
                                                    self.__class__.__name__))

    def getName(self):
        raise NotImplementedError(
            "{} not implemented in class {}".format(ModelInterpreter.getName.__name__, self.__class__.__name__))

    def getDependentModels(self):
        raise NotImplementedError(
            "{} not implemented in class {}".format(ModelInterpreter.getDependentModels.__name__,
                                                    self.__class__.__name__))


# Here we resemble the CreateModelInterpreterServicesVisitor

_model_interpreters = {}


def add_model_interpreter(library, model_interpreter):
    _model_interpreters[library] = model_interpreter


def get_model_interpreter(library_id) -> ModelInterpreter:
    if library_id not in _model_interpreters:
        raise ModelInterpreterNotFound("No interpreter found for library {}".format(library_id))
    return _model_interpreters[library_id]



def get_model_interpreter_from_library(library) -> ModelInterpreter:
    library_id = library.id
    return get_model_interpreter(library_id)


class ModelInterpreterNotFound(Exception): pass


def get_model_interpreter_from_variable(variable) -> ModelInterpreter:
    '''

    :param geppetto_model_type: EObject
    :return:
    '''
    library = variable.types[0].eContainer()
    return get_model_interpreter_from_library(library)

def get_model_interpreter_from_type(geppetto_model_type) -> ModelInterpreter:
    '''

    :param geppetto_model_type: EObject
    :return:
    '''
    library = geppetto_model_type.eContainer()
    return get_model_interpreter_from_library(library)



import sys

from pyecore.resources import ResourceSet
from pyecore.resources.json import JsonOptions
from pygeppetto.model import ISynchable

from .geppetto_resource import GeppettoResource
from .utils.bytesuri import BytesURI

GeppettoModelSerializer = GeppettoSerializer = sys.modules[__name__]

from pyecore.resources.json import DefaultObjectMapper


class SynchableMapper(DefaultObjectMapper):
    def __init__(self):
        pass

    def to_dict_from_obj(self, obj, options, use_uuid, resource):
        if hasattr(obj, 'synched'):
            if obj.synched:
                return {'synched': True}
            else:
                d = super().to_dict_from_obj(obj, options, use_uuid, resource)
                obj.synched = True
                return d
        else:
            return super().to_dict_from_obj(obj, options, use_uuid, resource)


class IgnoreSyncMapper(DefaultObjectMapper):
    def __init__(self):
        pass

    def to_dict_from_obj(self, obj, options, use_uuid, resource):
        if hasattr(obj, 'synched') and obj.synched:
            obj.synched = False
            d = super().to_dict_from_obj(obj, options, use_uuid, resource)
            obj.synched = True
            return d
        else:
            return super().to_dict_from_obj(obj, options, use_uuid, resource)


def serialize(geppetto_model, onlySerialiseDelta=False):
    # we now create a resource to save the geppetto model and serialize it to a JSON string
    rset = ResourceSet()
    uri = BytesURI('geppetto_model.json')
    rset.resource_factory['*'] = lambda uri: GeppettoResource(uri, indent=2)
    resource = rset.create_resource(uri)
    resource.append(geppetto_model)
    if onlySerialiseDelta:
        resource.register_mapper(ISynchable, SynchableMapper())
    else:
        resource.register_mapper(ISynchable, IgnoreSyncMapper())

    resource.save(options={JsonOptions.SERIALIZE_DEFAULT_VALUES: True})
    return uri.getvalue().decode("utf-8")


def load(json):
    rset = ResourceSet()
    uri = BytesURI('geppetto_model.json')
    rset.resource_factory['*'] = lambda uri: GeppettoResource(uri, indent=2)
    resource = rset.create_resource(uri)



from pygeppetto.model import ExperimentState


class RuntimeExperiment(object):
    def __init__(self, project, experiment):
        self.project = project
        self.experiment = experiment
        self.state = ExperimentState()

    def get_experiment_state(self, variables, url_base):
        pass



from pygeppetto.model import GeppettoModel, model_utility, World
from pygeppetto.model.exceptions import GeppettoModelException
from pygeppetto.model.model_factory import SharedLibraryManager
from pygeppetto.model.types import ImportType
from pygeppetto.model.utils import pointer_utility
from pyecore.commands import CommandStack, Add, EditingDomain


class GeppettoModelAccess:
    '''Gives access to a model and the common library'''

    def __init__(self, geppetto_model):
        if (type(geppetto_model) == str):
            geppetto_model = self.create_geppetto_model(geppetto_model)
        self.geppetto_model = geppetto_model
        try:
            self.geppetto_common_library = next(lib for lib in self.geppetto_model.libraries if lib.id == 'common')
        except StopIteration:
            self.geppetto_common_library = SharedLibraryManager.get_shared_common_library()
            geppetto_model.libraries.append(self.geppetto_common_library)

    @classmethod
    def create_geppetto_model(cls, name):
        # We create a GeppettoModel instance and we add the common library to it

        geppetto_model = GeppettoModel(name=name, libraries=[SharedLibraryManager.get_shared_common_library()])
        return geppetto_model

    def swap_type(self, itype: ImportType, newtype):
        # TODO test
        newtype.eContainer().synched = False
        for variable in itype.referencedVariables:
            variable.synched = False
            variable.types.clear()
            variable.types.append(newtype)

    def get_variable(self, variable_path):
        # TODO support worlds in get_variable
        return pointer_utility.find_variable_from_path(self.geppetto_model, variable_path)

    def get_type(self, type_path):
        return pointer_utility.get_type(self.geppetto_model, type_path)

    def get_value(self, variable_path):
        try:
            return self.get_variable(variable_path).initialValues[0].value
        except Exception as e:
            raise GeppettoModelException("Can't find a value for path " + variable_path) from e

    def get_query(self, query_path):
        return model_utility.get_query(model=self.geppetto_model, query_path=query_path)

    def add_variable_legacy(self, variable):
        # TODO Implement with commands: see https://pyecore.readthedocs.io/en/latest/user/advanced.html#modifying-elements-using-commands

        self.geppetto_model.variables.append(variable)
        self.geppetto_model.synched = False

    def add_variable(self, variable, world_id=None, legacy=False):
        # TODO Implement add_variable with commands: see https://pyecore.readthedocs.io/en/latest/user/advanced.html#modifying-elements-using-commands
        if legacy:
            return self.add_variable_legacy(variable)
        world = self.get_world(world_id)
        world.variables.append(variable)
        world.synched = False
        self.geppetto_model.synched = False

    def get_world(self, world_id=None) -> World:
        if len(self.geppetto_model.worlds) < 1:
            raise GeppettoModelException(f"No world was defined in the geppetto model")
        if len(self.geppetto_model.worlds) > 1:
            raise NotImplementedError("Multiple worlds are not yet supported")
        # TODO support multiple worlds
        if world_id is None:
            return self.geppetto_model.worlds[0]
        try:
            return next(world for world in self.geppetto_model.worlds if world.id == world_id)
        except StopIteration:
            raise GeppettoModelException(f"World not fount in model: {world_id}")

    def add_instance(self, instance, world_name=None):
        # TODO Implement add_instance with commands: see https://pyecore.readthedocs.io/en/latest/user/advanced.html#modifying-elements-using-commands
        world = self.get_world(world_name)
        world.instances.append(instance)
        world.synched = False
        self.geppetto_model.synched = False

    def get_variables(self, world_name=None):
        return self.get_world(world_name).variables

    def get_instances(self, world_name=None):
        return self.get_world(world_name).instances
